from __future__ import print_function

from distutils.version import LooseVersion
import json
import jsonschema
import os
import pkg_resources
import re


ValidationError = jsonschema.ValidationError


class SchemaNotFoundError(Exception):
    def __init__(self, kind, version, api_version):
        self.kind = kind
        self.version = version
        self.api_version = api_version
        self.message = ("Couldn't find schema for kind %s with api version %s for kubernetes version %s" %
                        (self.kind, self.api_version, self.version))


class InvalidSchemaError(Exception):
    def __init__(self, message):
        self.message = message


def all_versions():
    schemas = pkg_resources.resource_listdir('kubernetes_validate', '/kubernetes-json-schema')
    version_regex = re.compile(r'^v([^-]*).*')
    return sorted([version_regex.sub(r"\1", schema) for schema in schemas if version_regex.match(schema)],
                  key=LooseVersion)


def latest_version():
    return all_versions()[-1]


def validate(data, desired_version, strict=False):
    # strip initial v from version (I keep forgetting, so other people will too)
    if desired_version.startswith('v'):
        desired_version = desired_version[1:]
    # actual schema version is the latest version that is not newer than the desired version
    version = [version for version in all_versions() if version <= LooseVersion(desired_version)][-1]
    # Remove the trailing domain from the api version namespace and replace the / with -
    # e.g. rbac.authorization.k8s.io/v1 -> rbac-v1
    api_version = re.sub(r'^([^./]*)(?:\.[^/]*)?/', r'\1-', data['apiVersion'])
    schema_dir = 'v%s-local' % version
    if strict:
        schema_dir += '-strict'
    schema_file = pkg_resources.resource_filename('kubernetes_validate',
                                                  '/kubernetes-json-schema/%s/%s-%s.json' %
                                                  (schema_dir, data['kind'].lower(), api_version))

    try:
        f = open(schema_file)
    except IOError:
        raise SchemaNotFoundError(version=desired_version, kind=data['kind'], api_version=data['apiVersion'])
    try:
        schema = json.load(f)
    except json.JsonDecodeError:
        raise InvalidSchemaError("Couldn't parse schema %s" % schema_file)
    finally:
        f.close()
    schema_dir = os.path.dirname(os.path.abspath(schema_file))
    resolver = jsonschema.RefResolver(base_uri='file://' + schema_dir + '/', referrer=schema)

    try:
        jsonschema.validate(data, schema, resolver=resolver)
    except jsonschema.ValidationError:
        raise
    except jsonschema.exceptions.RefResolutionError:
        raise



#!/usr/bin/env python

from __future__ import print_function

import argparse
import errno
import sys
import yaml

import kubernetes_validate.utils as utils
from kubernetes_validate.version import __version__


def main():
    parser = argparse.ArgumentParser(description='validate a kubernetes resource definition')
    parser.add_argument('-k', '--kubernetes-version', action='append',
                        help='version of kubernetes against which to validate. Defaults to %s' %
                        utils.latest_version())
    parser.add_argument('--strict', action='store_true', default=False,
                        help='whether to use strict validation, rejecting unexpected properties')
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('filenames', nargs=argparse.REMAINDER)

    args = parser.parse_args()

    if not args.filenames:
        parser.print_help()
        return 0

    rc = 0
    for filename in args.filenames:
        try:
            f = open(filename)
        except Exception as e:
            raise SystemExit("Couldn't open file %s for reading: %s" % (filename, str(e)))
        try:
            data = yaml.load(f.read())
        except Exception as e:
            raise SystemExit("Couldn't parse YAML from file %s: %s" % (filename, str(e)))
        f.close()

        for version in args.kubernetes_version or [utils.latest_version()]:
            try:
                utils.validate(data, version, args.strict)
                print("INFO  %s passed against version %s" % (filename, version))
            except utils.ValidationError as e:
                print("ERROR %s did not validate against version %s: %s: %s" %
                      (filename, version, '.'.join([str(item) for item in e.path]), e.message))
                rc = 1
            except (utils.SchemaNotFoundError, utils.InvalidSchemaError) as e:
                print("ERROR %s" % e.message)
                rc = 2
    return rc


if __name__ == "__main__":
    try:
        sys.exit(main())
    except IOError as exc:
        if exc.errno != errno.EPIPE:
            raise
    except RuntimeError as e:
        raise SystemExit(str(e))



import os
from setuptools import setup, find_packages
import sys


sys.path.insert(0, os.path.abspath('lib'))

exec(open('src/kubernetes_validate/version.py').read())

setup(
    name='kubernetes-validate',
    version=__version__,
    description=('validates kubernetes resource definitions against schemas'),
    keywords='kubernetes, schema, validate, validator',
    author='Will Thames',
    url='https://github.com/willthames/kubernetes-validate',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    package_data={
        '': ['kubernetes-json-schema/*-local/*.json', 'kubernetes-json-schema/*-local-strict/*.json']
    },
    zip_safe=False,
    install_requires=['PyYAML', 'jsonschema'],
    entry_points={
        'console_scripts': [
             'kubernetes-validate=kubernetes_validate.__main__:main'
        ]
    },
    license='Apache',
    test_suite="test"
)
